-- --------------------
-- NETZWERK-DATENMODELL
-- --------------------

-- Netzwerk lschen
CALL SDO_NET.DROP_NETWORK('OldenburgNetz');

-- Netzwerk anlegen

-- SDO-Netzwerk anlegen:
DECLARE
BEGIN
  SDO_NET.CREATE_SDO_NETWORK('OldenburgNetz', 1, FALSE, FALSE);
END;
/

-- Dabei werden automatisch Tabellen und Indexe fr die Netzdaten angelegt:
SELECT table_name FROM USER_TABLES WHERE table_name LIKE 'OLDENBURGNETZ%';

SELECT table_name, index_name, index_type
FROM user_indexes
WHERE table_name LIKE 'OLDENBURGNETZ%';

SELECT * FROM USER_SDO_NETWORK_METADATA;

-- Daten laden

-- Aufruf des SQL*Loaders von der KOMMANDOZEILE aus:
SQLLDR USERID=<Benutzer>@<Dienstname> CONTROL=ol_knoten.ctl LOG=ol_knoten.log ERRORS=50
SQLLDR USERID=<Benutzer>@<Dienstname> CONTROL=ol_kanten.ctl LOG=ol_kanten.log ERRORS=50

-- Links mit Java-Anwendung ergnzen

-- Daten berprfen

SELECT node_id, n.geometry FROM OldenburgNetz_NODE$ n;

SELECT link_id, l.geometry FROM OldenburgNetz_LINK$ l;

-- Metadaten einfgen

INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_NODE$', 'GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5), SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),
  NULL
);
INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_LINK$', 'GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5), SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),
  NULL
);
INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_PATH$', 'GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5), SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),
  NULL
);
COMMIT;

-- rumliche Indexe anlegen

CREATE INDEX olnet_node_idx ON OldenburgNetz_NODE$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX olnet_link_idx ON OldenburgNetz_LINK$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX olnet_path_idx ON OldenburgNetz_PATH$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;


-- Krzeste Wege berechnen

-- evtl. zuvor existierende Pfade lschen:
DELETE FROM OldenburgNetz_PLINK$;
DELETE FROM OldenburgNetz_PATH$;
COMMIT;

-- krzeste Wege mit Java berechnen

-- Pfad berprfen:
SELECT path_id, cost, p.geometry FROM OldenburgNetz_PATH$ p;

SELECT SUM(l.cost)
FROM OldenburgNetz_PLINK$ pl INNER JOIN OldenburgNetz_LINK$ l
ON pl.link_id = l.link_id
GROUP BY path_id;


-- ---------------------
-- TOPOLOGIE-DATENMODELL
-- ---------------------

--- SQL-Zugriff
--- -----------

-- ggf. alte Daten lschen

CALL SDO_TOPO_MAP.DROP_TOPO_MAP('GeoDbLandTM');

CALL SDO_TOPO.DELETE_TOPO_GEOMETRY_LAYER('GeoDbLandTP', 'GeoDbLandNutzung','gebiet');
CALL SDO_TOPO.DROP_TOPOLOGY('GeoDbLandTP');
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'GEODBLANDNUTZUNG';
DELETE FROM GeoDbLandNutzung;
COMMIT;

DROP TABLE GeoDbLandNutzung;

-- Topology anlegen

CALL SDO_TOPO.CREATE_TOPOLOGY('GeoDbLandTP', 0.001, NULL);

-- Dabei werden automatisch Tabellen, Indexe, Sequenzgeneratoren und Metadaten
-- fr die Topologieprimitive angelegt:
SELECT topology, topology_id FROM USER_SDO_TOPO_METADATA;

SELECT table_name FROM USER_TABLES WHERE table_name LIKE 'GEODBLANDTP%';

SELECT table_name, index_name, index_type
FROM USER_INDEXES
WHERE table_name LIKE 'GEODBLANDTP%';

SELECT * FROM USER_SEQUENCES WHERE sequence_name LIKE 'GEODBLANDTP%';

SELECT * FROM USER_SDO_GEOM_METADATA WHERE table_name LIKE 'GEODBLANDTP%';

-- Ausdehnung der Geo-Metadaten korrigieren:
UPDATE USER_SDO_GEOM_METADATA
SET diminfo = SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('X',0,20,0.001),
  SDO_DIM_ELEMENT('Y',0,20,0.001) )
WHERE table_name LIKE 'GEODBLANDTP%';
COMMIT;

-- Tabelle fr SDO_TOPO_GEOMETRY-Objekte anlegen:
CREATE TABLE GeoDbLandNutzung (
  id       INTEGER,                         -- ID
  nutzung  VARCHAR(20),                     -- Nutzungsart
  gebiet   SDO_TOPO_GEOMETRY,               -- Topology Geometry
  CONSTRAINT pk_gdblandnutz PRIMARY KEY(id) -- Primrschlssel
);

-- Tabelle bekanntgeben:
CALL SDO_TOPO.ADD_TOPO_GEOMETRY_LAYER('GeoDbLandTP', 'GeoDbLandNutzung','gebiet', 'POLYGON');
-- GEODBLANDTP_RELATION$ wurde dadurch angelegt:
SELECT table_name FROM USER_TABLES WHERE table_name = 'GEODBLANDTP_RELATION$';
-- Metadaten abfragen
SELECT topology_id, table_name, column_name, tg_layer_id, tg_layer_type, srid
FROM USER_SDO_TOPO_METADATA
WHERE topology = 'GEODBLANDTP';

-- Daten einfgen

-- Einfgen in Knotentabelle:
INSERT INTO GeoDbLandTP_Node$
VALUES (GeoDbLandTP_Node_S.nextval,1,NULL, SDO_GEOMETRY('POINT(1 1)'));

COMMIT;

-- Einfgen in Maschentabelle:
INSERT INTO GeoDbLandTP_Face$ VALUES (-1,NULL,SDO_LIST_TYPE(-1),SDO_LIST_TYPE(),NULL);

INSERT INTO GeoDbLandTP_Face$
VALUES (1,1,SDO_LIST_TYPE(),SDO_LIST_TYPE(),SDO_GEOMETRY(2003,NULL,NULL,
                        SDO_ELEM_INFO_ARRAY(1,1003,3),SDO_ORDINATE_ARRAY(1,1,19,20)));
COMMIT;


-- Einfgen in Kantentabelle:
INSERT INTO GeoDbLandTP_Edge$
VALUES (1, 1,1, 1,1,-1,-1, 1,-1,
        SDO_GEOMETRY('LINESTRING(1 1,19 1,19 20,1 20,1 1)'));
COMMIT;

-- Geoobjekt einfgen:
INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (6,'Weideland',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                        SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3))) );
COMMIT;
        
-- berprfen:
SELECT * FROM GeoDbLandTP_RELATION$;

-- Geometrie erzeugen:
SELECT nutzung, l.gebiet.get_geometry() FROM GeoDbLandNutzung l;

-- Metadaten aktualisieren:
CALL SDO_TOPO.INITIALIZE_METADATA('GeoDbLandTP');
-- Sequenzgeneratoren berprfen
SELECT sequence_name, last_number FROM USER_SEQUENCES WHERE sequence_name LIKE 'GEODBLANDTP%S';


--- PL/SQL-Zugriff
--- --------------

-- berprfung der Korrektheit ohne (Fehler-) Meldung:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPOLOGY('GeoDbLandTP');
END;
/

-- berprfung der Korrektheit mit Fehlermeldung:
UPDATE GeoDbLandTP_Face$ SET face_id = 2 WHERE face_id = 1;

DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPOLOGY('GeoDbLandTP');
END;
/

ROLLBACK;

-- TopoMap anlegen:
CALL SDO_TOPO_MAP.CREATE_TOPO_MAP('GeoDbLandTP','GeoDbLandTM');

-- TopoMap laden:
-- CREATE TABLE Protokoll (text VARCHAR(1024));

DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.LOAD_TOPO_MAP('GeoDbLandTM',0,0,20,20,'TRUE');
  -- DELETE FROM Protokoll;
  -- INSERT INTO Protokoll VALUES (res);
  -- COMMIT;
END;
/

-- Knoten und Schlinge einfgen:
DECLARE 
  id NUMBER(38);     -- ID
BEGIN
  id := SDO_TOPO_MAP.ADD_ISOLATED_NODE(NULL,1, SDO_GEOMETRY('POINT(3 10)'));
  id := SDO_TOPO_MAP.ADD_LOOP(NULL,id,SDO_GEOMETRY('LINESTRING(3 10, 4 5, 9 3, 13 5, 16 8, 17 12, 13 15, 7 14, 3 10)'));
END;
/

-- Korrektheit berprfen:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPO_MAP('GeoDbLandTM');
END;
/

-- TopoMap in die Datenbank zurckschreiben:
CALL SDO_TOPO_MAP.COMMIT_TOPO_MAP();

-- Masche berprfen:
SELECT face_id, boundary_edge_id, mbr_geometry FROM GeoDbLandTP_FACE$ ORDER BY face_id;
-- Kante berprfen:
SELECT edge_id, left_face_id, right_face_id FROM GeoDbLandTP_EDGE$ ORDER BY edge_id;
-- Knoten berprfen
SELECT * FROM GeoDbLandTP_NODE$ ORDER BY node_id;

-- berprfen der Verknpfungen:
SELECT * FROM GeoDbLandTP_RELATION$;

-- Geoobjekt ndern und einfgen:
UPDATE GeoDbLandNutzung
SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3)))
WHERE nutzung = 'Weideland';

INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (8,'Wasser',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                        SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(3,3))) );
COMMIT;

-- Geometrien berprfen
SELECT nutzung, l.gebiet.get_geometry() FROM GeoDbLandNutzung l;

-- TopoMap freigeben:
CALL SDO_TOPO_MAP.CLEAR_TOPO_MAP('GeoDbLandTM');

-- TopoMap neu laden:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.LOAD_TOPO_MAP('GeoDbLandTM',0,0,20,20,'TRUE');
END;
/

-- Knoten und Kante einfgen:
DECLARE 
  id4 NUMBER(38);     -- Knoten-ID von n4
  id5 NUMBER(38);     -- Knoten-ID von n5
  id  NUMBER(38);     -- ID neue Kante
BEGIN
  id4 := SDO_TOPO_MAP.ADD_NODE(NULL,1,SDO_GEOMETRY('POINT(19 12)'), 1,'TRUE');
  id5 := SDO_TOPO_MAP.ADD_NODE(NULL,3,SDO_GEOMETRY('POINT(17 12)'), 5,'FALSE');
  id := SDO_TOPO_MAP.ADD_EDGE(NULL, id4,id5, SDO_GEOMETRY('LINESTRING(19 12, 17 12)'));
END;
/

-- Knoten und Kante einfgen:
DECLARE 
  id3 NUMBER(38);     -- Knoten-ID von n3
  id6 NUMBER(38);     -- Knoten-ID von n6
  id  NUMBER(38);     -- ID neue Kante
BEGIN
  id3 := 3;
  id6 := SDO_TOPO_MAP.ADD_NODE(NULL,4,SDO_GEOMETRY('POINT(1 10)'), 2,'TRUE');
  id := SDO_TOPO_MAP.ADD_EDGE(NULL, id3,id6, SDO_GEOMETRY('LINESTRING(3 10, 1 10)'));
END;
/

-- Korrektheit berprfen:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPO_MAP('GeoDbLandTM');
END;
/

-- TopoMap in die Datenbank zurckschreiben:
CALL SDO_TOPO_MAP.COMMIT_TOPO_MAP();

-- Maschen berprfen:
SELECT face_id, mbr_geometry FROM GeoDbLandTP_FACE$ ORDER BY face_id;
-- Kante berprfen:
SELECT edge_id, left_face_id, right_face_id, geometry FROM GeoDbLandTP_EDGE$ ORDER BY edge_id;
-- Knoten berprfen
SELECT * FROM GeoDbLandTP_NODE$ ORDER BY node_id;

-- Geoobjekt ndern und einfgen:
UPDATE GeoDbLandNutzung
SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3)))
WHERE nutzung = 'Weideland';

INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (9,'Wald',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                        SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(4,3))) );
COMMIT;

-- Geometrien berprfen:
SELECT nutzung, l.gebiet.get_geometry() FROM GeoDbLandNutzung l;

-- TopoMap lschen:
CALL SDO_TOPO_MAP.DROP_TOPO_MAP('GeoDbLandTM');


--- Java-Zugriff
--- ------------

-- Die Topologien und Objekte ber das Java-Programm eingefgen.

-- Topologische Geoobjekt 'Wasser' korrigieren:
UPDATE GeoDbLandNutzung
SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(3,3)))
WHERE nutzung = 'Wasser';

COMMIT;

-- Maschen berprfen:
SELECT face_id, mbr_geometry FROM GeoDbLandTP_FACE$ ORDER BY face_id;

-- Geometrien berprfen:
SELECT id, nutzung, l.gebiet.get_geometry() FROM GeoDbLandNutzung l;



--- Anfragen
--- --------

-- SDO_ANYINTERACT:

-- Topologietest: bergabe einer SDO_TOPO_GEOMETRY:
SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_ANYINTERACT (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

-- Topologietest: bergabe einer SDO_GEOMETRY:
SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLand haus
ON SDO_ANYINTERACT (n.gebiet, haus.geo) = 'TRUE'
WHERE haus.name = 'Haus';

-- spezifische Operatoren:

-- Topologietest: bergabe einer SDO_TOPO_GEOMETRY:
SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_TOUCH (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_EQUAL (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

-- Topologietest: bergabe einer SDO_GEOMETRY:
SELECT id, nutzung
FROM GeoDbLandNutzung
WHERE SDO_CONTAINS (gebiet, SDO_GEOMETRY('POINT(3 3)')) = 'TRUE';

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLand haus
ON SDO_EQUAL (n.gebiet, haus.geo) = 'TRUE'
WHERE haus.name = 'Haus';

SELECT id, nutzung
FROM GeoDbLandNutzung n
WHERE SDO_COVERS (gebiet, SDO_GEOMETRY(2002,NULL,NULL, SDO_ELEM_INFO_ARRAY(1,2,1),
                         SDO_ORDINATE_ARRAY(9,9, 11,9.7))) = 'TRUE';

-- RELATE zur Bestimmung der Beziehung:
SELECT n1.id, n1.nutzung, n2.id, n2.nutzung, SDO_TOPO.RELATE(n1.gebiet,n2.gebiet,'DETERMINE')
FROM GeoDbLandNutzung n1 INNER JOIN GeoDbLandNutzung n2
ON n1.id < n2.id
WHERE n2.id <> 15;

-- SDO_TOPO_OBJECT_ARRAY als 2. Parameter:
SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_ANYINTERACT (n.gebiet, wasser.gebiet.GET_TOPO_ELEMENTS()) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

