-- -------------------
-- SPATIAL DATA MINING
-- -------------------

-- Aufrumen:
DROP TABLE Clusters;
DELETE FROM USER_SDO_GEOM_METADATA WHERE table_name = 'CLUSTERS';
COMMIT;

-- Cluster-Tabelle anlegen:
CREATE TABLE Clusters (
  id   DECIMAL(3),                       -- ID des Clusters
  geo  SDO_GEOMETRY,                     -- Gebiet des Clusters
  CONSTRAINT pk_cluster PRIMARY KEY (id) -- Primrschlssel
);

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('CLUSTERS','GEO', SDO_DIM_ARRAY(SDO_DIM_ELEMENT('Lnge',-180,180,1),
                                        SDO_DIM_ELEMENT('Breite',-90,90,1)) ,
        8307);
COMMIT;

-- Rumlichen Index anlegen:
CREATE INDEX Clusters_geo_ix
ON Clusters(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Version ohne 'Outliers'

-- Cluster-Bestimmung:
INSERT INTO Clusters (id,geo)
  SELECT * FROM TABLE(SDO_SAM.SPATIAL_CLUSTERS('GEMEINDEN','CENTRUM',10,'TRUE'));

COMMIT;

SELECT id, geo FROM Clusters;

SELECT geo FROM Clusters UNION ALL SELECT centrum FROM Gemeinden;

SELECT SUM(SDO_GEOM.SDO_AREA(c.geo,1,'unit=SQ_KM')) FROM Clusters c;

-- Geometrien auerhalb der Cluster bestimmen:
SELECT *
FROM (SELECT g.name, MIN( SDO_GEOM.SDO_DISTANCE (c.geo, g.centrum, 1, 'unit=M') ) AS dist
      FROM Gemeinden g CROSS JOIN Clusters c
      GROUP BY g.name)
WHERE Dist > 0;

-- Funktion zur fehlerfreien Zuordnung zu Clustern:
CREATE OR REPLACE FUNCTION BIN_GEOMETRY (
  data_geom IN SDO_GEOMETRY, tolerance IN NUMBER, 
  cluster_tablename IN VARCHAR2, cluster_colname IN VARCHAR2) RETURN NUMBER
IS
BEGIN
  RETURN SDO_SAM.BIN_GEOMETRY(data_geom,tolerance,cluster_tablename,cluster_colname);
EXCEPTION WHEN OTHERS THEN
  RETURN -1;
END;
/

-- Cluster-Zuordnung:
SELECT name, BIN_GEOMETRY(centrum,1,'CLUSTERS','GEO')
FROM Gemeinden
ORDER BY NAME;

-- Version mit 'Outliers'

DELETE FROM Clusters;

-- Cluster-Bestimmung:
INSERT INTO Clusters (id,geo)
  SELECT * FROM TABLE(SDO_SAM.SPATIAL_CLUSTERS('GEMEINDEN','CENTRUM',10,'FALSE'));

COMMIT;

SELECT id, geo FROM Clusters;

SELECT geo FROM Clusters UNION ALL SELECT centrum FROM Gemeinden;

SELECT SUM(SDO_GEOM.SDO_AREA(c.geo,1,'unit=SQ_KM')) FROM Clusters c;

-- Geometrien auerhalb der Cluster bestimmen:
SELECT *
FROM (SELECT g.name, MIN( SDO_GEOM.SDO_DISTANCE (c.geo, g.centrum, 1, 'unit=M') ) AS dist
      FROM Gemeinden g CROSS JOIN Clusters c
      GROUP BY g.name)
WHERE Dist > 0;

-- Cluster-Zuordnung:
SELECT name, BIN_GEOMETRY(centrum,1,'CLUSTERS','GEO')
FROM Gemeinden
ORDER BY NAME;
